@@ -7,54 +7,56 @@
 #
 # Written by Andy Wardley <abw@wardley.org>
 #
-# Copyright (C) 1997-2003 Andy Wardley.  All Rights Reserved.
+# Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
 # Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
-#
-# $Id: AppConfig.pm,v 1.7 2004/02/04 10:28:28 abw Exp $
-#
-#============================================================================
+#==========================================================================
 
 package AppConfig;
 
 use strict;
-
-require 5.005;
-
-## This is the main version number for AppConfig
-## It is extracted by ExtUtils::MakeMaker and inserted in various places.
-use vars qw{$VERSION @ISA };
-BEGIN {
-	require Exporter;
-	$VERSION = '1.63';
-	@ISA     = qw{ Exporter };
-}
+use warnings;
+use base 'Exporter';
+our $VERSION = 1.65;
 
 # variable expansion constants
-use constant EXPAND_NONE    => 0;
-use constant EXPAND_VAR     => 1;
-use constant EXPAND_UID     => 2;
-use constant EXPAND_ENV     => 4;
-use constant EXPAND_ALL     => EXPAND_VAR | EXPAND_UID | EXPAND_ENV;
-use constant EXPAND_WARN    => 8;
+use constant EXPAND_NONE   => 0;
+use constant EXPAND_VAR    => 1;
+use constant EXPAND_UID    => 2;
+use constant EXPAND_ENV    => 4;
+use constant EXPAND_ALL    => EXPAND_VAR | EXPAND_UID | EXPAND_ENV;
+use constant EXPAND_WARN   => 8;
 
 # argument count types
-use constant ARGCOUNT_NONE  => 0;
-use constant ARGCOUNT_ONE   => 1;
-use constant ARGCOUNT_LIST  => 2;
-use constant ARGCOUNT_HASH  => 3;
+use constant ARGCOUNT_NONE => 0;
+use constant ARGCOUNT_ONE  => 1;
+use constant ARGCOUNT_LIST => 2;
+use constant ARGCOUNT_HASH => 3;
 
 # Exporter tagsets
-my @EXPAND   = qw(EXPAND_NONE EXPAND_VAR EXPAND_UID EXPAND_ENV 
-                  EXPAND_ALL EXPAND_WARN);
-my @ARGCOUNT = qw(ARGCOUNT_NONE ARGCOUNT_ONE ARGCOUNT_LIST ARGCOUNT_HASH);
+our @EXPAND = qw(
+    EXPAND_NONE
+    EXPAND_VAR
+    EXPAND_UID
+    EXPAND_ENV 
+    EXPAND_ALL
+    EXPAND_WARN
+);
 
-use vars qw( $AUTOLOAD @EXPORT_OK %EXPORT_TAGS );
-@EXPORT_OK   = (@EXPAND, @ARGCOUNT);
-%EXPORT_TAGS = (
+our @ARGCOUNT = qw(
+    ARGCOUNT_NONE
+    ARGCOUNT_ONE
+    ARGCOUNT_LIST
+    ARGCOUNT_HASH
+);
+
+our @EXPORT_OK   = ( @EXPAND, @ARGCOUNT );
+our %EXPORT_TAGS = (
     expand   => [ @EXPAND   ],
     argcount => [ @ARGCOUNT ],
 );
+our $AUTOLOAD;
 
+require AppConfig::State;
 
 #------------------------------------------------------------------------
 # new(\%config, @vars)
@@ -66,16 +68,9 @@ use vars qw( $AUTOLOAD @EXPORT_OK %EXPORT_TAGS );
 
 sub new {
     my $class = shift;
-
-    require AppConfig::State;
-
-    my $self = {
+    bless {
         STATE => AppConfig::State->new(@_)
-    };
-
-    bless $self, $class;
-	
-    return $self;
+    }, $class;
 }
 
 
@@ -170,7 +165,6 @@ sub cgi {
     $cgi->parse(shift);
 }
 
-    
 #------------------------------------------------------------------------
 # AUTOLOAD
 #
@@ -179,6 +173,7 @@ sub cgi {
 # AppConfig::State object.
 #
 #------------------------------------------------------------------------
+
 sub AUTOLOAD {
     my $self = shift;
     my $method;
@@ -193,7 +188,6 @@ sub AUTOLOAD {
     $self->{ STATE }->$method(@_);
 }
 
-
 1;
 
 __END__
@@ -205,36 +199,39 @@ AppConfig - Perl5 module for reading configuration files and parsing command lin
 =head1 SYNOPSIS
 
     use AppConfig;
-
+    
     # create a new AppConfig object
-    my $config = AppConfig->new(\%cfg);
-
+    my $config = AppConfig->new( \%cfg );
+    
     # define a new variable
-    $config->define($varname => \%varopts);
-
+    $config->define( $varname => \%varopts );
+    
     # create/define combined
-    my $config = AppConfig->new(\%cfg, 
-    $varname => \%varopts, $varname => \%varopts, ...);
-
+    my $config = AppConfig->new( \%cfg, 
+        $varname => \%varopts,
+        $varname => \%varopts,
+        ...
+    );
+    
     # set/get the value
-    $config->set($varname, $value);
+    $config->set( $varname, $value );
     $config->get($varname);
-
+    
     # shortcut form
     $config->varname($value);
-    $config->varname();
-
+    $config->varname;
+    
     # read configuration file
     $config->file($file);
-
+    
     # parse command line options
-    $config->args(\@args);	# default to \@ARGV
-
+    $config->args(\@args);      # default to \@ARGV
+    
     # advanced command line options with Getopt::Long
-    $config->getopt(\@args);	# default to \@ARGV
-
+    $config->getopt(\@args);    # default to \@ARGV
+    
     # parse CGI parameters (GET method)
-    $config->cgi($query);	# default to $ENV{ QUERY_STRING }
+    $config->cgi($query);       # default to $ENV{ QUERY_STRING }
 
 =head1 OVERVIEW
 
@@ -252,14 +249,14 @@ by its configuration when defined.
     verbose 
     nohelp
     debug = On
-
+    
     # single value
     home  = /home/abw/
-
+    
     # multiple list value
     file = /tmp/file1
     file = /tmp/file2
-
+    
     # multiple hash value
     book  camel = Programming Perl
     book  llama = Learning Perl
@@ -287,7 +284,7 @@ Configuration files may be arranged in blocks as per the style of Win32
     src  = ~/websrc/docs/$site
     lib  = ~/websrc/lib
     dest = ~/public_html/$site
-
+    
     [page]
     header = $lib/header
     footer = $lib/footer
@@ -299,7 +296,7 @@ text in a configuration file.
     line 1
     line 2
     FOOBAR
-
+    
     paths  exe  = "${PATH}:${HOME}/.bin"
     paths  link = <<'FOO'
     ${LD_LIBARRAY_PATH}:${HOME}/lib
@@ -322,9 +319,9 @@ CGI script via the URL (GET method).
 
 AppConfig requires Perl 5.005 or later.  
 
-The Getopt::Long and Test::More modules should be installed.  If you
-are using a recent version of Perl (e.g. 5.8.0) then these should 
-already be installed.
+The L<Getopt::Long> and L<Test::More> modules should be installed.
+If you are using a recent version of Perl (e.g. 5.8.0) then these
+should already be installed.
 
 =head1 OBTAINING AND INSTALLING THE AppConfig MODULE BUNDLE
 
@@ -334,9 +331,9 @@ manual page explains:
     CPAN stands for the Comprehensive Perl Archive Network.
     This is a globally replicated collection of all known Perl
     materials, including hundreds of unbundled modules.  
-
+    
     [...]
-
+    
     For an up-to-date listing of CPAN sites, see
     http://www.perl.com/perl/ or ftp://ftp.perl.com/perl/ .
 
@@ -390,7 +387,7 @@ AppConfig modules.
 
 =head2 USING THE AppConfig MODULE
 
-To import and use the AppConfig module the following line should 
+To import and use the L<AppConfig> module the following line should 
 appear in your Perl script:
 
      use AppConfig;
@@ -420,14 +417,14 @@ to the AppConfig::State constructor.
 The first (optional) parameter may be a reference to a hash array
 containing configuration information.  
 
-    my $config = AppConfig->new({
-	    CASE   => 1,
-	    ERROR  => \&my_error,
-	    GLOBAL => { 
-		    DEFAULT  => "<unset>", 
-		    ARGCOUNT => ARGCOUNT_ONE,
-		},
-	});
+    my $config = AppConfig->new( {
+            CASE   => 1,
+            ERROR  => \&my_error,
+            GLOBAL => { 
+                    DEFAULT  => "<unset>", 
+                    ARGCOUNT => ARGCOUNT_ONE,
+                },
+        } );
 
 See L<AppConfig::State> for full details of the configuration options
 available.  These are, in brief:
@@ -464,7 +461,7 @@ Subsequent parameters may be variable definitions.  These are passed
 to the define() method, described below in L<DEFINING VARIABLES>.
 
     my $config = AppConfig->new("foo", "bar", "baz");
-    my $config = AppConfig->new({ CASE => 1 }, qw(foo bar baz));
+    my $config = AppConfig->new( { CASE => 1 }, qw(foo bar baz) );
 
 Note that any unresolved method calls to AppConfig are automatically 
 delegated to the AppConfig::State object.  In practice, it means that
@@ -473,7 +470,7 @@ AppConfig::State object:
 
     # create AppConfig
     my $config = AppConfig->new('foo', 'bar');
-
+    
     # methods get passed through to internal AppConfig::State
     $config->foo(100);
     $config->set('bar', 200);
@@ -497,19 +494,19 @@ reference to a hash array may also be passed to specify configuration
 information for the variable:
 
     $config->define("foo", {
-	    DEFAULT   => 99,
-	    ALIAS     => 'metavar1',
-	});
+            DEFAULT   => 99,
+            ALIAS     => 'metavar1',
+        });
 
 Configuration items specified in the GLOBAL option to the module 
 constructor are applied by default when variables are created.  e.g.
 
-    my $config = AppConfig->new({ 
-	    GLOBAL => {
-		    DEFAULT  => "<undef>",
-		    ARGCOUNT => ARGCOUNT_ONE,
-		}
-	});
+    my $config = AppConfig->new( { 
+        GLOBAL => {
+            DEFAULT  => "<undef>",
+            ARGCOUNT => ARGCOUNT_ONE,
+        }
+    } );
     
     $config->define("foo");
     $config->define("bar", { ARGCOUNT => ARGCOUNT_NONE } );
@@ -517,16 +514,16 @@ constructor are applied by default when variables are created.  e.g.
 is equivalent to:
 
     my $config = AppConfig->new();
-
-    $config->define("foo", {
-		DEFAULT  => "<undef>",
-		ARGCOUNT => ARGCOUNT_ONE,
-	    });
-
-    $config->define("bar", 
-		DEFAULT  => "<undef>",
-		ARGCOUNT => ARGCOUNT_NONE,
-	    });
+    
+    $config->define( "foo", {
+        DEFAULT  => "<undef>",
+        ARGCOUNT => ARGCOUNT_ONE,
+    } );
+    
+    $config->define( "bar", 
+        DEFAULT  => "<undef>",
+        ARGCOUNT => ARGCOUNT_NONE,
+    } );
 
 Multiple variables may be defined in the same call to define().
 Configuration hashes for variables can be omitted.
@@ -565,11 +562,13 @@ ARGCOUNT_LIST, C</%/> = ARGCOUNT_HASH, C</[=:].*/> = ARGCOUNT_ONE)
 =item EXPAND
 
 Specifies which variable expansion policies should be used when parsing 
-configuration files.  Constants in C<:expand> tag set define EXPAND_NONE
-- no expansion (default), EXPAND_VAR - expand C<$var> or C<$(var)> as 
-other AppConfig variables, EXPAND_UID - expand C<~uid> as user's home 
-directory, EXPAND_ENV - expand C<${var}> as environment variable,
-EXPAND_ALL - do all expansions.  May be logically or'd.
+configuration files.  Constants in C<:expand> tag set define:
+
+    EXPAND_NONE - no expansion (default) 
+    EXPAND_VAR  - expand C<$var> or C<$(var)> as other variables
+    EXPAND_UID  - expand C<~> and C<~uid> as user's home directory 
+    EXPAND_ENV - expand C<${var}> as environment variable
+    EXPAND_ALL - do all expansions. 
 
 =item VALIDATE
 
@@ -609,36 +608,35 @@ The following examples demonstrate use of the comapct format, with their
 equivalent full specifications:
 
     $config->define("foo|bar|baz!");
-
+    
     $config->define(
-	    "foo" => { 
-		ALIAS    => "bar|baz", 
-		ARGCOUNT => ARGCOUNT_NONE,
-	    });
-
+            "foo" => { 
+                ALIAS    => "bar|baz", 
+                ARGCOUNT => ARGCOUNT_NONE,
+            });
+    
     $config->define("name=s");
-
+    
     $config->define(
-	    "name" => { 
-		ARGCOUNT => ARGCOUNT_ONE,
-	    });
-
+            "name" => { 
+                ARGCOUNT => ARGCOUNT_ONE,
+            });
+    
     $config->define("file|filelist|f=s@");
-
+    
     $config->define(
-	    "file" => { 
-		ALIAS    => "filelist|f", 
-		ARGCOUNT => ARGCOUNT_LIST,
-	    });
-
-
+            "file" => { 
+                ALIAS    => "filelist|f", 
+                ARGCOUNT => ARGCOUNT_LIST,
+            });
+    
     $config->define("user|u=s%");
-
+    
     $config->define(
-	    "user" => { 
-		ALIAS    => "u", 
-		ARGCOUNT => ARGCOUNT_HASH,
-	    });
+            "user" => { 
+                ALIAS    => "u", 
+                ARGCOUNT => ARGCOUNT_HASH,
+            });
 
 Additional configuration options may be specified by hash reference, as per 
 normal.  The compact definition format will override any configuration 
@@ -747,8 +745,8 @@ a '\'.
     # this is a comment
     callsign = alpha bravo camel delta echo foxtrot golf hipowls \
                india juliet kilo llama mike november oscar papa  \
-	       quebec romeo sierra tango umbrella victor whiskey \
-	       x-ray yankee zebra
+               quebec romeo sierra tango umbrella victor whiskey \
+               x-ray yankee zebra
 
 Variables that are simple flags and do not expect an argument (ARGCOUNT = 
 ARGCOUNT_NONE) can be specified without any value.  They will be set with 
@@ -799,7 +797,7 @@ A reference to the hash is returned when the variable is requested.
 
     my $aliases = $config->alias();
     foreach my $k (keys %$aliases) {
-	print "$k => $aliases->{ $k }\n";
+        print "$k => $aliases->{ $k }\n";
     }
 
 The '-' prefix can be used to reset a variable to its default value and
@@ -819,10 +817,10 @@ Three different expansion types may be applied:
 
     bin = ~/bin          # expand '~' to home dir if EXPAND_UID
     tmp = ~abw/tmp       # as above, but home dir for user 'abw'
-
+    
     perl = $bin/perl     # expand value of 'bin' variable if EXPAND_VAR
     ripl = $(bin)/ripl   # as above with explicit parens
-
+    
     home = ${HOME}       # expand HOME environment var if EXPAND_ENV
 
 See L<AppConfig::State> for more information on expanding variable values.
@@ -836,7 +834,7 @@ file.
 
     [block1]
     foo = 10             # block1_foo = 10
-
+    
     [block2]
     foo = 20             # block2_foo = 20
 
@@ -893,13 +891,13 @@ is encountered.
 
     myprog -file /tmp/foo -file /tmp/bar # $config->file('/tmp/foo')
                                          # $config->file('/tmp/bar')
-
+    
     # file => [ '/tmp/foo', '/tmp/bar' ]
     
     myprog -door "jim=Jim Morrison" -door "ray=Ray Manzarek"
                                     # $config->door("jim=Jim Morrison");
                                     # $config->door("ray=Ray Manzarek");
-
+    
     # door => { 'jim' => 'Jim Morrison', 'ray' => 'Ray Manzarek' }
 
 See L<AppConfig::Args> for further details on parsing command line
@@ -927,10 +925,10 @@ specification string is constructed from the name, any aliases (delimited
 by a vertical bar '|') and the value of the ARGS parameter.
 
     $config->define("foo", {
-	ARGS  => "=i",
-	ALIAS => "bar|baz",
+        ARGS  => "=i",
+        ALIAS => "bar|baz",
     });
-
+    
     # Getopt::Long specification: "foo|bar|baz=i"
 
 Errors and warning generated by the Getopt::Long module are trapped and 
@@ -955,9 +953,9 @@ equals sign:
 
     http://www.here.com/cgi-bin/myscript?foo=bar&baz=qux&verbose
 
-	# $config->foo('bar');
-	# $config->baz('qux');
-	# $config->verbose(1);
+        # $config->foo('bar');
+        # $config->baz('qux');
+        # $config->verbose(1);
 
 Certain values specified in a URL must be escaped in the appropriate 
 manner (see CGI specifications at http://www.w3c.org/ for full details).  
@@ -965,14 +963,14 @@ The AppConfig::CGI module automatically unescapes the CGI query string
 to restore the parameters to their intended values.
 
     http://where.com/mycgi?title=%22The+Wrong+Trousers%22
-
-	# $config->title('"The Wrong Trousers"');
+    
+    # $config->title('"The Wrong Trousers"');
 
 Please be considerate of the security implications of providing writeable
 access to script variables via CGI.
 
     http://rebel.alliance.com/cgi-bin/...
-	.../send_report?file=%2Fetc%2Fpasswd&email=darth%40empire.com
+        .../send_report?file=%2Fetc%2Fpasswd&email=darth%40empire.com
 
 To avoid any accidental or malicious changing of "private" variables, 
 define only the "public" variables before calling the cgi() (or any 
@@ -980,10 +978,10 @@ other) method.  Further variables can subequently be defined which
 can not be influenced by the CGI parameters.
 
     $config->define('verbose', 'debug')
-    $config->cgi();		# can only set verbose and debug
+    $config->cgi();             # can only set verbose and debug
 
     $config->define('email', 'file');
-    $config->file($cfgfile);	# can set verbose, debug, email + file
+    $config->file($cfgfile);    # can set verbose, debug, email + file
 
 
 =head1 CONSTANT DEFINITIONS
@@ -1036,13 +1034,9 @@ With contributions from Dave Viner, Ijon Tichy, Axel Gerstmair and
 many others whose names have been lost to the sands of time (reminders
 welcome).
 
-=head1 REVISION
-
-Revision $Revision: 1.7 $ distributed as part of AppConfig 1.56.
-
 =head1 COPYRIGHT
 
-Copyright (C) 1997-2004 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1997-2007 Andy Wardley.  All Rights Reserved.
 
 Copyright (C) 1997,1998 Canon Research Centre Europe Ltd.
 
